#!/usr/bin/env bash

SCRIPT_PATH=$(cd $(dirname "$(readlink -f "$0")")/ && pwd)
source "$SCRIPT_PATH"/common.sh # load commons

LOG_FILE="$LOGS_DIR"/$(basename "$0" .sh).log

DEFAULT_DOCKER_OS=centos
DEFAULT_AM2CM_DOCKER_IMAGE_VERSION=latest
DEFAULT_AM2CM_SERVER_PROFILE=prod
DEFAULT_PYTHON_EXECUTABLE=python3

DOCKER_OS=${DEFAULT_DOCKER_OS}
AM2CM_DOCKER_IMAGE_VERSION=${DEFAULT_AM2CM_DOCKER_IMAGE_VERSION}
AM2CM_SERVER_PROFILE=${DEFAULT_AM2CM_SERVER_PROFILE}

for i in "$@"; do
  case $i in
  --local)
    SETUP_LOCAL=true
    ;;
  --docker)
    SETUP_DOCKER=true
    ;;
  -os=* | --docker-os=*)
    DOCKER_OS=${i#*=}
    ;;
  -iv=* | --image-version=*)
    AM2CM_DOCKER_IMAGE_VERSION=${i#*=}
    ;;
  -p=* | --profile=*)
    AM2CM_SERVER_PROFILE=${i#*=}
    ;;
  -py=* | --python-executable=*)
    PYTHON_EXECUTABLE=${i#*=}
    ;;
  -h | --help)
    HELP=true
    ;;
  *)
    echo "Unknown option: $i"
    HELP=true
    ;;
  esac
done

__usage="
 Usage: $(basename "$0") [options]
 Options:
    --local: Install am2cm on the local machine
    --docker: Install am2cm in a docker container
    -os, --docker-os: the OS of the docker container (Default: $DEFAULT_DOCKER_OS)
    -p, --profile: Set the active profile(s) for the am2cm-server (Default: $DEFAULT_AM2CM_SERVER_PROFILE)
    -py, --python-executable: Path to python3 executable
    -h, --help: Print this help
 "

if [ "${HELP}" = true ]; then
  echo "$__usage"
  exit 0
fi

while [ -z "${SETUP_LOCAL}" ] && [ -z "${SETUP_DOCKER}" ]; do
  read -r -p "Select the type of installation ([l]ocal|[d]ocker):" INSTALL_TYPE
  case  $INSTALL_TYPE in
    l | local) SETUP_LOCAL=true;;
    d | docker) SETUP_DOCKER=true;;
    *) echo "ERROR: The '${INSTALL_TYPE}' is not a valid installation type! Please select one of the following options [l]ocal or [d]ocker"
      ;;
  esac
done

if [ "${SETUP_DOCKER}" = true ]; then
  if [ -z "${DOCKER_OS}" ]; then
    read -r -p "Enter the OS of the docker container [${DEFAULT_DOCKER_OS}]:" DOCKER_OS
    DOCKER_OS=${DOCKER_OS:-${DEFAULT_DOCKER_OS}}
  fi
  if [ -z "${AM2CM_DOCKER_IMAGE_VERSION}" ]; then
    read -r -p "Enter the version for the docker image [${DEFAULT_AM2CM_DOCKER_IMAGE_VERSION}]:" AM2CM_DOCKER_IMAGE_VERSION
    AM2CM_DOCKER_IMAGE_VERSION=${AM2CM_DOCKER_IMAGE_VERSION:-${DEFAULT_AM2CM_DOCKER_IMAGE_VERSION}}
  fi
fi

if [ -z "${AM2CM_SERVER_PROFILE}" ]; then
  read -r -p "Enter the active profile for the am2cm-server [${DEFAULT_AM2CM_SERVER_PROFILE}]:" AM2CM_SERVER_PROFILE
  AM2CM_SERVER_PROFILE=${AM2CM_SERVER_PROFILE:-$DEFAULT_AM2CM_SERVER_PROFILE}
fi

if [ "${SETUP_DOCKER}" = true ]; then
  echo "Building docker image..." | tee -a "$LOG_FILE"
  "$SCRIPT_PATH"/build-docker.sh $DEV_OPTS -os="${DOCKER_OS}" -iv="${AM2CM_DOCKER_IMAGE_VERSION}" --profile="${AM2CM_SERVER_PROFILE}" | tee -a "$LOG_FILE"
fi

if [ "${SETUP_LOCAL}" = true ]; then

  cleanup() {
    if ! delete_virtual_env; then
      echo "The ${VENV_DIR} must be deleted manually!"
    fi
    exit 1
  }

  download_software_dependencies_for_offline_use() {
    source "${VENV_DIR}"/bin/activate
    if [ "$VIRTUAL_ENV" = "" ]; then
      echo "Virtualenv could not be activated in ${VENV_DIR}" | tee -a "$LOG_FILE"
      exit 1;
    fi
    dependency_install_playbooks=( download_jdbc_drivers download_python_for_offline_use download_python_packages_for_offline_use )
    cd "${ROOT_DIR}"/am2cm-ansible
    for i in "${dependency_install_playbooks[@]}"
    do
     ansible-playbook -e "@inventories/group_vars/all.yml" -e "cma_root_dir=$ROOT_DIR" "playbooks/install/$i.yml" | tee -a "$LOG_FILE"
    done
    deactivate
  }

  trap cleanup SIGINT

  if [ -z "${PYTHON_EXECUTABLE}" ]; then
    PYTHON_EXECUTABLE=$(get_python_executable $DEFAULT_PYTHON_EXECUTABLE)
  fi

  echo "Creating virtualenv in ${VENV_DIR} ..." | tee -a "$LOG_FILE"
  "${PYTHON_EXECUTABLE}" -m venv "${VENV_DIR}" | tee -a "$LOG_FILE"

  echo "Installing requirements for am2cm-ansible in ${VENV_DIR} ..." | tee -a "$LOG_FILE"
  "${VENV_DIR}"/bin/pip install -r "${ROOT_DIR}"/am2cm-ansible/requirements.txt  | tee -a "$LOG_FILE"

  echo "Download software dependencies for offline use.." | tee -a "${LOG_FILE}"
  download_software_dependencies_for_offline_use

  echo 'Configure am2cm-server ...' | tee -a "$LOG_FILE"
  cat >"${ROOT_DIR}"/am2cm-server/.env <<EOF
export AM2CM_ROOT="${ROOT_DIR:-$PWD}"
export ACTIVE_PROFILE=${AM2CM_SERVER_PROFILE}
EOF
fi

echo "Setup completed!" | tee -a "$LOG_FILE"